package com.example.waves_sdk

import android.util.Log
import com.wavesplatform.sdk.WavesSdk
import com.wavesplatform.sdk.utils.RxUtil
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.reactivex.disposables.CompositeDisposable

class WavesNodeServiceHandler {

  fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {

    if (call.method == "addressesBalance") {
      getWavesBalance(call.arguments(), result)
      return
    }

    result.notImplemented()
  }

  private val compositeDisposable = CompositeDisposable()

  private fun getWavesBalance(address: String, result: MethodChannel.Result) {
    compositeDisposable.add(
      WavesSdk.service()
        .getNode()
        .addressesBalance(address)
        .compose(RxUtil.applyObservableDefaultSchedulers())
        .subscribe({ wavesBalance ->
          result.success(wavesBalance.balance)
        }, { error ->
          result.error("ERROR", "Can't get wavesBalance", error.message)
          Log.d("LOG", "Can't get wavesBalance! + ${error.message}")
        })
    )
  }
}
