import 'dart:typed_data';
import 'package:flutter/services.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:waves_sdk/waves_sdk.dart';

void main() {
  MethodChannel('waves_sdk').setMockMethodCallHandler((MethodCall methodCall) async {});

  MethodChannel('waves_crypto').setMockMethodCallHandler((MethodCall methodCall) async {
    switch (methodCall.method) {
      case 'randomSeed': return 'seed test';
      case 'addressBySeed': return 'address';
      case 'publicKey': return 'publicKey';
      case 'privateKey': return 'privateKey';
      case 'keyPair': return {'publicKey': 'publicKey', 'privateKey': 'privateKey'};
      case 'addressByPublicKey': return 'address';
      case 'signBytesWithPrivateKey': return 123;
    }
    return methodCall.method;
  });

  MethodChannel('waves_node_service').setMockMethodCallHandler((MethodCall methodCall) async {
    if (methodCall.method == 'addressesBalance') {
      return 100;
    }
    return methodCall.method;
  });

  String seed;
  String address;
  String publicKey;
  String privateKey;

  group('WavesCrypto', () {
    test('Init SDK', () async {
      await WavesSdk.init(env: Environment.TEST_NET);
      expect(WavesSdk.ready, true);
    });

    test('randomSeed', () async {
      seed = await WavesCrypto.randomSeed();
      expect(seed, isNotNull);
    });

    test('addressBySeed', () async {
      address = await WavesCrypto.addressBySeed(seed);
      expect(address, isNotNull);
    });

    test('publicKey', () async {
      publicKey = await WavesCrypto.publicKey(seed);
      expect(publicKey, isNotNull);
    });

    test('privateKey', () async {
      privateKey = await WavesCrypto.privateKey(seed);
      expect(privateKey, isNotNull);
    });

    test('keyPair', () async {
      var value = await WavesCrypto.keyPair(seed);
      expect(value, isNotNull);
      expect(value.publicKey, publicKey);
      expect(value.privateKey, privateKey);
    });

    test('addressByPublicKey', () async {
      var addressByPublicKey = await WavesCrypto.addressByPublicKey(publicKey);
      expect(addressByPublicKey, isNotNull);
      expect(addressByPublicKey, address);
    });

    test('signBytesWithPrivateKey', () async {
      var result = await WavesCrypto.signBytesWithPrivateKey(Uint8List(100), privateKey);
      expect(result, isNotNull);
    });
  });

  group('Node Service', () {
    test('Init SDK', () async {
      await WavesSdk.init(env: Environment.TEST_NET);
      expect(WavesSdk.ready, true);
    });

    test('addressesBalance', () async {
      String balance = await WavesSdk.service().getNode().addressesBalance(address);
      expect(balance, isNotNull);
      expect(balance, '100.00000000');
    });
  });
}
